#ifndef SUBDIVISION_H
#define SUBDIVISION_H

#include <QtOpenGL>
#include <QOpenGLFunctions_4_3_Core>

#include "mesh.h"

class Subdivision
{
public:
    Subdivision(QOpenGLFunctions_4_3_Core *f);
    ~Subdivision();

    void init();
    Mesh *subdivide(Mesh *mesh);
    void setDebugOutbut(boolean debug);

private:
    boolean debugOutput;

    struct Input
    {
        GLuint vb_handle;
        QVector<unsigned int> index_buffer;
        QVector<Vertex> vertex_buffer;
    };

    struct Tables
    {
        QVector<GLuint> edge_indices;
        QVector<GLuint> vertex_indices;
        QVector<GLuint> vertex_offsets;
        QVector<GLuint> index_buffer;
    };

    struct Result
    {
      GLuint vb_handle;
      QVector<Vertex> vertex_buffer;
    };

    QOpenGLFunctions_4_3_Core *f;
    QOpenGLShaderProgram *edgeShader;
    QOpenGLShaderProgram *vertexShader;

    QOpenGLShaderProgram *initComputeShaderProgram(QString &source);
    Tables precomputeTables(Input input);
    Result runShader(Input input, Tables &tables);
    void runVertexShader(GLuint size, GLuint vb_handle, GLuint vertex_indices_handle, GLuint vertex_offsets_handle, GLuint output_handle);
    void runEdgeShader(GLuint size, GLuint vb_handle, GLuint edge_indices_handle, GLuint output_handle, GLuint offset);

};

#endif
